@@ -23,11 +23,14 @@ perldl or pdl2 shell.
 
 package PDL::Dbg;
 
+use strict;
+use warnings;
 # used by info
-$PDL::Dbg::Title = "Type   Dimension       Flow  State          Mem";
-$PDL::Dbg::Infostr = "%6T %-15D  %3F   %-5S  %12M";
+our $Title = "Type   Dimension       State          Mem";
+our $Infostr = "%6T %-15D  %-5S  %12M";
 
-package PDL;
+package # hide from MetaCPAN
+  PDL;
 
 =head1 FUNCTIONS
 
@@ -112,27 +115,22 @@ sub px {
   my $classname = $arg;
   # find the correct package
   $package .= "::" unless $package =~ /::$/;
-  *stab = *{"main::"};
-  while ($package =~ /(\w+?::)/g){
-    *stab = $ {stab}{$1};
-  }
+  my $stab = \%main::;
+  $stab = $stab->{$_.'::'} for grep length, split /::/, $package;
   print "$classname variables in package $package\n\n";
   my $title = "Name         $PDL::Dbg::Title\n";
   print $title;
   print '-'x(length($title)+3)."\n";
-  my ($pdl,$npdls,$key,$val,$info) = ((),0,"","","");
-  # while (($key,$val) = each(%stab)) {
-  foreach $key ( sort { lc($a) cmp lc($b) } keys(%stab) ) {
-     $val = $stab{$key};
-     $pdl = ${"$package$key"};
+  my $npdls = 0;
+  foreach my $key ( sort { lc($a) cmp lc($b) } keys %$stab ) {
+     my $pdl = do { no strict 'refs'; ${"$package$key"} };
+     next if !UNIVERSAL::isa($pdl,$classname);
      # print info for all objects derived from this class
-     if (UNIVERSAL::isa($pdl,$classname)) {
-        $npdls++;
-        $info = $pdl->info($PDL::Dbg::Infostr);
-        printf "\$%-11s %s %s\n",$key,$info,(ref($pdl) eq $classname ? '' :
-           ref($pdl));
-        # also print classname for derived classes
-     }
+     $npdls++;
+     my $info = $pdl->info($Infostr);
+     printf "\$%-11s %s %s\n",$key,$info,(ref $pdl eq $classname ? '' :
+        ref $pdl);
+     # also print classname for derived classes
   }
   print "no $classname objects in package $package\n"
        unless $npdls;
